/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: sdp.h,v 1.10 2002/09/10 13:46:38 kds Exp $

   Top level SDP definitions for attribute types. 
   This file provides a "one-stop" change for any
   modifications to the attributes

   Fixes:
		Dmitry Kasatkin		: bug fixes, cleanup, re-arrangement, 
					  continuation state, mtu
*/

#ifndef SDP_H
#define SDP_H

#include <stdint.h>
#include <glib.h>

#include <affix/bluetooth.h>
#include <affix/utils.h>

/*
 ** All definitions are based on Bluetooth Assigned Numbers
 ** of the Bluetooth Specification
 **
 **
 ** NOTE : The quotes("") are needed as this value is treated
 ** as a string in uuidHandler.c
 **
 ** NOTE : The conversion of this string to 128bit relies
 ** in preserving the sequence shown below ie. if you need
 ** to change, then preserve the "-" characters in the 
 ** appropriate places. If their positions CHANGE, then you need to
 ** MODIFY THE CODE makeBluetoothBaseUUID() in uuidHandler.c
 **
 */
#define BASE_UUID "00000000-0000-1000-8000-00805F9B34FB"

/*
 ** Protocol and service multiplexor (PSM) standards
 */
#define SDP_PSM 	 	0x0001
#define RFCOMM_PSM  	 	0x0003
#define TCS_BIN_PSM 	 	0x0005
#define TCS_BIN_CORDLESS_PSM 	0x0007
#define BNEP_PSM		0x000F

/*
 ** Protocol UUIDs
 */
#define SDP_UUID	0x0001
#define UDP_UUID	0x0002
#define RFCOMM_UUID	0x0003
#define TCP_UUID	0x0004
#define TCS_BIN_UUID	0x0005
#define TCS_AT_UUID	0x0006
#define OBEX_UUID	0x0008
#define IP_UUID		0x0009
#define FTP_UUID        0x000A
#define HTTP_UUID	0x000C
#define WSP_UUID	0x000E
#define BNEP_UUID	0x000F
#define L2CAP_UUID	0x0100

/*
 ** Service class identifiers of standard services and
 ** service groups
 */
#define SDPServerServiceClassID      		0x1000
#define BrowseGroupDescServiceClassID 		0x1001
#define PublicBrowseGroup                       0x1002
#define SerialPortServiceClassID     		0x1101
#define LANAccessUsingPPPServiceClassID     	0x1102
#define DialupNetworkingServiceClassID     	0x1103
#define IrMCSyncServiceClassID     		0x1104
#define ObexObjectPushServiceClassID     	0x1105
#define ObexFileTransferServiceClassID     	0x1106
#define IrMCSyncCommandServiceClassID     	0x1107
#define HeadsetServiceClassID     		0x1108
#define CordlessTelephonyServiceClassID  	0x1109
#define IntercomServiceClassID  		0x1110
#define FaxServiceClassID  			0x1111
#define HeadsetAudioGatewayServiceClassID  	0x1112
#define PnPInformationServiceClassID  		0x1200
#define GenericNetworkingServiceClassID  	0x1201
#define GenericFileTransferServiceClassID  	0x1202
#define GenericAudioServiceClassID  		0x1203
#define GenericTelephonyServiceClassID  	0x1204
/* new */
#define HandsFreeServiceClassID			0x111E
#define AGHandsFreeServiceClassID		0x111F
#define PANUServiceClassID			0x1115
#define NAPServiceClassID			0x1116
#define GNServiceClassID			0x1117


/*
 ** Attribute identifier codes
 */
#define SDPServerServiceRecordHandle 0x0000

/*
 ** Possible values for ServiceAttributeIdentifier are
 ** listed below. See SDP Spec, section "Service Attribute
 ** Definitions" for more details.
 */
#define AttrServiceRecordHandle         0x0000
#define AttrServiceClassIDList          0x0001
#define AttrServiceRecordState          0x0002
#define AttrServiceID                   0x0003
#define AttrProtocolDescriptorList      0x0004
#define AttrBrowseGroupList             0x0005
#define AttrLanguageBasedAttrIDList     0x0006
#define AttrServiceInfoTimeToLive       0x0007
#define AttrServiceAvailability         0x0008
#define AttrProfileDescriptorList     0x0009
#define AttrDocumentationURL            0x000A
#define AttrClientExecutableURL         0x000B
/*
 ** There is a conflict with the SDP spec and
 ** assigned numbers doc in which this attribute
 ** has a value of 0x000D
 */
#define AttrIconURL                     0x000C                      

/*
 ** These identifiers are based on the SDP
 ** spec stating that "base attribute id of
 ** the primary (universal) language must be 0x0100
 */
#define PrimaryLangBase 		0x0100

#define AttrServiceName_Primary         	0x0000+PrimaryLangBase
#define AttrServiceDescription_Primary  	0x0001+PrimaryLangBase
#define AttrServiceProviderName_Primary 	0x0002+PrimaryLangBase
/*
 ** Other languages should have theor own offset ex.
 ** #define XXXLangBase yyyy
 ** #define AttrServiceName_XXX	0x0000+XXXLangBase
 ** ..
 */

/*
 ** These attributes are specific to the SDP server only
 ** ie. can be present only in the ServiceRecord of the
 ** SDP server
 */
#define AttrVersionNumberList           0x0200
#define AttrServiceDatabaseState        0x0201           

#define AttrGroupID			0x0200

/* HandsFree */
#define AttrSupportedFeatures		0x0311

/* PAN */
#define AttrSecurityDescription		0x030A
#define AttrNetAccessType		0x030B
#define AttrMaxNetAccessRate		0x030C

/* --------------------------------------------------------------- */

/*
 ** The Data representation in SDP PDUs (pg 339 and 340 of BT SDP Spec)
 **
 ** These are the exact data type+size descriptor values
 ** that go into PDU buffer.
 **
 ** The datatype(leading 5bits) + size descriptor(last 3 bits)
 ** is 8 bits. The size descriptor is critical to extract the
 ** right number of bytes for the data value from the PDU.
 **
 ** For most basic types, the datatype+size descriptor is
 ** straightforward. However for constructed types and strings,
 ** the size of the data is in the next "n" bytes following the
 ** 8 bits (datatype+size) descriptor. Exactly what the "n" is
 ** specified in the 3 bits of the data size descriptor.
 **
 ** TextString and URLString can be of size 2^{8, 16, 32} bytes
 ** DataSequence and DataSequenceAlternates can be of size 2^{8, 16, 32}
 ** The size are computed post-facto in the API and are not known apriori
 */
/*
 ** Permitted values for DataTypeDescriptor
 */
#define Data_Nil 			0x00
#define UnsignedInteger_8  		0x08
#define UnsignedInteger_16		0x09
#define UnsignedInteger_32		0x0A
#define UnsignedInteger_64		0x0B
#define UnsignedInteger_128		0x0C
#define Integer_8			0x10
#define Integer_16			0x11
#define Integer_32			0x12
#define Integer_64			0x13
#define Integer_128			0x14
#define UUID_16				0x19
#define UUID_32				0x1A
#define UUID_128			0x1C
#define TextStringSize8			0x25
#define TextStringSize16		0x26
#define TextStringSize32        	0x27
#define Boolean				0x28
#define SequenceSize8			0x35
#define SequenceSize16			0x36
#define SequenceSize32			0x37
#define AlternativeSize8		0x3D
#define AlternativeSize16		0x3E
#define AlternativeSize32		0x3F
#define URLStringSize8			0x45
#define URLStringSize16			0x46
#define URLStringSize32			0x47


/*
 ** Our own internal data types
 */
#define UUIDId				0x76

/*
 ** The PDU identifiers of SDP packets between client
 ** and server
 */
#define SDP_ErrorResponse			0x01
#define SDP_ServiceSearchRequest 		0x02
#define SDP_ServiceSearchResponse 		0x03
#define SDP_ServiceAttributeRequest 		0x04
#define SDP_ServiceAttributeResponse 		0x05
#define SDP_ServiceSearchAttributeRequest 	0x06
#define SDP_ServiceSearchAttributeResponse 	0x07


/*
 ** Some additions to support service registration.
 ** These are outside the scope of the Bluetooth specification
 */
#define SDP_ServiceRegistrationRequest 		0x75
#define SDP_ServiceRegistrationResponse 	0x76
#define SDP_ServiceUpdateRequest		0x77
#define SDP_ServiceUpdateResponse		0x78
#define SDP_ServiceRemovalRequest		0x79
#define SDP_ServiceRemovalResponse		0x80


/*
 ** SDP Error codes
 */
#define InvalidSDPVersion		0x0001
#define InvalidServiceRecordHandle	0x0002
#define InvalidRequestSyntax		0x0003
#define InvalidPDUSize			0x0004
#define InvalidContinutationState	0x0005


#define RequestBufferSize 2048
#define ResponseBufferSize USHRT_MAX    

/*
 ** Common return value definitions. Used by both 
 ** SDP service discovery clients and SDP service 
 ** registration clients and server-side components.
 */
#define WaitTimeForResponse 20

#define E_OK 		0
#define E_FAILURE 	-1
#define E_TIMEOUT 	-2
#define E_INVALID_ARGS 	-3
#define E_NOT_LOCAL 	-4
#define E_CONNTIMEDOUT	-5
#define E_CONNREFUSED	-6

/*
 ** Common definitions for attributes in the SDP.
 ** Should the type of any of these change, you need
 ** make a change only here. The rest of the code should
 ** work OK with the new type
 */

typedef uint16_t 	ServiceAttributeIdentifier;
typedef uint32_t 	ServiceRecordHandle;
typedef uint8_t 	DataTypeDescriptor;
typedef uint32_t 	ServiceInfoTimeToLive;
typedef uint8_t  	ServiceAvailability;
typedef uint16_t 	VersionNumber;
typedef uint16_t 	PortNumber;
typedef uint32_t 	ServiceRecordState;
typedef uint32_t 	ServiceDatabaseState;
typedef uint16_t 	LanguageCodeISO639;
typedef uint16_t 	CharacterEncoding;
typedef uint16_t 	LanguageBaseOffset;
typedef uint8_t 	DataSizeDescriptor8;
typedef uint16_t 	DataSizeDescriptor16;
typedef uint32_t 	DataSizeDescriptor32;
typedef uint16_t 	ErrorCode;
typedef int		SDPServerHandle;


typedef struct int128bit {
	char data[16];
} Int128Bit ;

typedef uint16_t 	UUID16Bit;
typedef uint32_t	UUID32Bit;
typedef Int128Bit	UUID128Bit;

typedef enum uuidType {
	bits_16 = UUID_16,
	bits_32 = UUID_32,
	bits_128 = UUID_128
} UUIDType;

typedef struct uuid {
	UUIDType uuidType;
	union {
		UUID16Bit uuid16Bit;
		UUID32Bit uuid32Bit;
		UUID128Bit uuid128Bit;
	} value ;
} UUID;      

/*
 ** Basic protocol descriptor - contains UUID of the
 ** protocol to use ex. L2CAP, RFCOMM etc
 ** VersionNumber and PortNumber are the other
 ** parameters of interest as defined in the SDP spec
 **
 ** Port numbers can sometimes be 2 bytes (RFCOMM SDP
 ** record). To support this, an additional "portNumberSize"
 ** field is added, which must be set to 1 or 2 bytes
 ** as the case may be. If not set, it defaults to 2
 */

typedef struct {
	DataTypeDescriptor	type;
	

} ProtocolParameter;

typedef struct protocolDescriptor {
	UUID		protocolId;
	PortNumber	portNumber;
	int		portNumberSize;
	VersionNumber	versionNumber;
} ProtocolDescriptor ;

/*
 ** A service could be accessed by a layer of protocols -
 ** RFCOMM over L2CAP
 ** PPP over RFCOMM over L2CAP
 ** A sequence of such protocols starting with the
 ** uppermost to the lowermost is present in one sequence
 **
 ** Typicaly GSList ** points to an array of one entry, however ..
 ** If a service can be accessed using more than one sequence
 ** then the number is specified by "alternates" and the alternate
 ** sequences are present in GSList **
 */
typedef struct accessProtocols {
	GSList *pSequence;
} AccessProtocols ;

/*
 ** User visible strings can be in many languages
 ** in addition to the universal language.
 **
 ** Language meta-data includes language code in
 ** ISO639 followed by the encoding format.
 ** The third field in this structure is the
 ** attribute offset for the language. User visible
 ** strings in the specified language can be obtained
 ** at this offset
 */
typedef struct languageAttributes {
	LanguageCodeISO639 langCodeISO639;
	CharacterEncoding charEncoding;
	LanguageBaseOffset langBaseOffset;
} LanguageAttributes ;

/*
 ** Profile descriptor is the Bluetooth profile meta-
 ** data. If a service conforms to a well-known profile,
 ** then it's profile identifier (UUID) is an attribute
 ** of the service. In addition if the profile has a version
 ** number it is specified here
 */
typedef struct profileDescriptor {
	UUID profileUUID;
	VersionNumber vnum;
} ProfileDescriptor ;

/* ------------------------------------------------------- */




typedef enum operationMode {
	client,
	server
} OperationMode;

/*
 ** Request blob as received from the client
 */
typedef struct sdpRequest {
	int	requestSocket;
	char	*requestBuffer;
} SDPRequest ;

/*
 ** Proprietary (Nokia) extensions to register/delete/modify
 ** service records
 */
typedef struct sdpPDUHeader {
	/*
	 ** PDU Id, should be one of SDP_XXXID
	 ** defined above
	 */
	uint8_t pduId;

	/*
	 ** Identity of the transaction
	 */
	uint16_t transactionId;

	/*
	 ** PDU parameter length, this does not
	 ** include the pduId and transaction id
	 */
	uint16_t paramLength;

} __PACK__ SdpPduHeader;

/*
 ** The service record when encapsulated as a PDU
 ** as per BT spec
 **
 **  Each of the PDU form objects are cached to
 ** avoid repetitive encodings on the server side
 ** They are however set to null on the client side
 */
typedef struct PduForm {
	char	*data;
	int	dataSize;
	int	bufferSize;
} PDUForm;     


typedef struct sdpVersionNumber {
	uint8_t	major;
	uint8_t	minor;
} SDPServerVersionNumber ;

/* ---------------------------------------------------- */



typedef enum serviceRecordInternalState {

	/*
	 ** Service record just created
	 */
	created,

	/*
	 ** Committed in SDP server. Implies
	 ** that the service record has all
	 ** the mandatory attributes set
	 */
	registered,

	/*
	 ** 
	 */
	modifiedUpdateReady,

	/*
	 ** This is the state when insufficient information
	 ** regarding the service is available (one or more
	 ** mandatory attributes are not set yet)
	 */
	createdRegisterReady,

	/*
	 ** Service record does not exist
	 */
	doesNotExist
} ServiceRecordInternalState ;

/*
 ** ServiceRecord is a composite of many attributes not
 ** all of which need be present. References of those
 ** composites not present would be set to NULL
 **
 ** A set of mandatory attributes will be defined and
 ** when set, the ServiceRecord gets into a "commit ready"
 ** state
 **
 */
typedef struct serviceRecord {

	ServiceRecordHandle      serviceRecordHandle;

	/*
	 ** Bluetooth address of the device that owns this
	 ** service. 
	 */
	BD_ADDR *bd_addr;

	/*
	 ** Set this flag to true when all mandatory attributes
	 ** are set, else it is false. If mandatory attributes
	 ** are not set, then service is not registrable
	 */
	ServiceRecordInternalState 	internalState;

	/*
	 ** The faimiliar PDU form and metadata of the entire 
	 ** service record. This is recomputed every time
	 ** an attribute changes
	 */
	PDUForm pduForm;

	/*
	 ** Search pattern .. a sequence of all possible
	 ** UUIDs seen in this record
	 */
	GSList *targetPattern;

	GSList *attributeList;
	
	int	ownerSocket;	/* to remove service */

} ServiceRecord ;


/*
 ** Create a service record. Empty for starts
 */
ServiceRecord *createServiceRecord(void);

/*
 ** Free a service record and it's attributes
 */
void freeServiceRecord(ServiceRecord *svcRec);
void freeServiceRecordList(GSList *svcRecList);


/*
 ** Free the "protocol descriptor" attribute of a service
 ** record
 */
void freeSList(GSList *pDesc);

/*
 ** Print the contents of a service record
 */
void printServiceRecord(BD_ADDR bdAddr, ServiceRecordHandle svcRecHandle);


/* -------------------------------------- */

typedef struct sdpData SDPData;
struct sdpData {
	DataTypeDescriptor		dtd;
	ServiceAttributeIdentifier	attrId;
	union {
		int8_t 		int8;
		int16_t 	int16;
		int32_t 	int32;
		int64_t 	int64;
		Int128Bit 	int128;
		uint8_t 	uint8;
		uint16_t 	uint16;
		uint32_t 	uint32;
		uint64_t 	uint64;
		Int128Bit 	uint128;
		UUID		uuid;
		char 		*stringPtr;
		GSList	 	*dataSeq;
	} sdpDataValue;
	PDUForm 	pduForm;
	int 		unitSize;
};

int extractAttributes(ServiceRecord *svcRec, char *localSeqPtr, int sequenceLength);
ServiceRecord *clientExtractPDU(char *pdata, ServiceRecordHandle handleExpected, int *bytesScanned);

SDPData *extractString(char *, int *);
void printServiceAttributes(GSList *pTable);
void printSDPData(SDPData *pSDPData);
void printDataSeq(GSList *dataSeq);
void freeSDPData(SDPData *pData);
void freeSDPDataSeq(GSList *pDataSeq);


SDPData *getSDPData(ServiceRecord *svcRec, ServiceAttributeIdentifier attrId);

void printSDPDataAsServiceRecord(ServiceRecord *svcRec);

#define BasicAttributePduFormSize 32
#define SequencePduFormSize 	  128
#define UUIDSequenceSize	  256

guint attrIdHashFunc(gconstpointer key);
gint attrIdCompareFunc(gconstpointer key1, gconstpointer key2);
void attrPrintFunc(gpointer value, gpointer userData); 



#endif //SDP_H
