/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: sdpclt.h,v 1.6 2002/09/20 21:26:47 kds Exp $

   SDP service discovery client API definitions

   Fixes:
		Dmitry Kasatkin		: bug fixes, cleanup, re-arrangement, 
					  continuation state, mtu
*/
	 
#ifndef SDP_CLT_H
#define SDP_CLT_H

#include <stdio.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include <affix/bluetooth.h>
#include <affix/btcore.h>

#include <affix/sdp.h>
//#include "record.h"

#define E_OK 0
#define E_NOT_EXIST -1


/*
** SDP Initialization procedure. Sets up SDP internal
** data structures. Must be called before any other
** SDP function can be invoked.
**
** INPUT :
**
**   NONE
**
** OUTPUT :
** 
**   E_OK : Initialization success
**
**   E_FAILURE : Initialization failure
**
*/
#define SDP_SVC_PROVIDER	0x01
int SDPInit(int flags);
void SDPCleanup(void);

/*
** Create a connection (L2CAP) to the specified
** Bluetooth device (BD_ADDR). 
**
** Note : The identification of the device on which
**        the SDP server is known to run is outside 
**        the scope of this function. For a utility
**        function to do this, see sdp_utils.h
**
** INPUT :
**  
**  char * bdAddr :
**    Address of the device where the SDP server is known
**    to run. In the format xx:yy:zz:ll:mm
**
**  SDPServerHandle :
**    Contains a handle to the SDP server on the remote
**    device. All subsequent transactions must be carried
**    out using this
**
*/
SDPServerHandle SDPOpenServerConnection(BD_ADDR bdAddr);
void SDPCloseServerConnection(SDPServerHandle srvHandle);

/*
** Find all devices in the piconet.
**
*/
int SDPGeneralInquiry(INQUIRY_ITEM *pItem, int numberOfDevices, int duration, uint8_t *numOfDevices);


/*
** Get the local device's Bluetooth (BD_ADDR)
** address
*/
BD_ADDR SDPGetLocalBDAddr(void);



/*
** This is a service search request. 
**
** INPUT :
**
**   GSList *svcSearchList
**     Singly linked list containing elements of the search
**     pattern. Each entry in the list is a UUID (DataTypeUUID_16)
**     of the service to be searched
**
**     Use the makeUUID16BitFromXXBit() macros for conversion
**     from 32 and 128 bits to 16 bits
**
**   uint16_t maxSvcRecordCount
**      A 16 bit integer which tells the service, the maximum
**      entries that the client can handle in the response. The
**      server is obliged not to return > maxSvcRecordCount entries
**
** OUTPUT :
**
**   int return value
**     E_OK 
**       The request completed successfully. This does not
**       mean the requested services were found
**     E_TIMEOUT 
**       The request completed unsuccessfully due to a timeout
**
**   GSList **svcHandleList
**     This variable is set on a successful return if there are
**     non-zero service handles. It is a singly linked list of
**     service record handles (ServiceRecordHandle)
**
**   uint16_t *handleCount
**     This is a pointer to a 16 bit integer, which is set to 
**     indicate the number of service record handles present in
**     svcHandleList
*/
int __SDPServiceSearchRequest(
  BD_ADDR bdAddr,
  //SDPServerHandle srvHandle,
  GSList *svcSearchList, 
  uint16_t maxSvcRecordCount,
  GSList **svcResponseList,
  uint16_t *handleCount
);


/*
** This is a service attribute request. 
**
** INPUT :
**
**   ServiceRecordHandle svcHandle
**     The handle of the service for which the attribute(s) are
**     requested
**
**   AttributeRequestType attrReqType
**     Attribute identifiers are 16 bit unsigned integers specified
**     in one of 2 ways described below :
**     IndividualAttributes - 16bit individual identifiers
**        They are the actual attribute identifiers in ascending order
**
**     RangeOfAttributes - 32bit identifier range
**        The high-order 16bits is the start of range
**        the low-order 16bits are the end of range
**        0x0000 to 0xFFFF gets all attributes
**
**   GSList *attrIDList
**     Singly linked list containing attribute identifiers desired.
**     Every element is either a uint16_t(attrSpec = IndividualAttributes)  
**     or a uint32_t(attrSpec=RangeOfAttributes)
**
**   uint16_t maxAttrIDByteCount
**     The byte count of the number of attribute IDs specified
**     in the request list
**
** OUTPUT :
**   int return value
**     E_OK 
**       The request completed successfully. This does not
**       mean the requested services were found
**     E_TIMEOUT 
**       The request completed unsuccessfully due to a timeout
**
**   uint16_t *maxAttrResponseByteCount
**     This is a pointer to a 16 bit integer, which is set to 
**     indicate the number of bytes of attributes returned. 
**     This pointer is set on successful return
**
*/

typedef enum attributeRequestType {
	/*
	 ** Attributes are specified as individual elements
	 */
	IndividualAttributes=1,
	/*
	 ** Attributes are specified as a range
	 */
	RangeOfAttributes
} AttributeRequestType;


int __SDPServiceAttributeRequest(
		BD_ADDR bdAddr,
		//SDPServerHandle srvHandle,
		ServiceRecordHandle svcHandle,
		AttributeRequestType attrReqType,
		GSList *attrIDList,
		uint16_t maxAttrIDByteCount,
		ServiceRecord	**_svcRec,
		uint16_t *maxAttrResponseByteCount
		);


/*
** This is a service search request combined with the service
** attribute request. First a service class match is done and
** for matching service, requested attributes are extracted
**
** INPUT :
**
**   GSList *svcSearchList
**     Singly linked list containing elements of the search
**     pattern. Each entry in the list is a UUID(DataTypeUUID_16)
**     of the service to be searched
**
**   AttributeSpecification attrSpec
**     Attribute identifiers are 16 bit unsigned integers specified
**     in one of 2 ways described below :
**     IndividualAttributes - 16bit individual identifiers
**        They are the actual attribute identifiers in ascending order
**
**     RangeOfAttributes - 32bit identifier range
**        The high-order 16bits is the start of range
**        the low-order 16bits are the end of range
**        0x0000 to 0xFFFF gets all attributes
**
**   GSList *attrIDList
**     Singly linked list containing attribute identifiers desired.
**     Every element is either a uint16_t(attrSpec = IndividualAttributes)  
**     or a uint32_t(attrSpec=RangeOfAttributes)
**
**   uint16_t maxAttrIDByteCount
**     The byte count of the number of attribute IDs specified
**     in the request list
**
** OUTPUT :
**   int return value
**     E_OK 
**       The request completed successfully. This does not
**       mean the requested services were found
**     E_TIMEOUT 
**       The request completed unsuccessfully due to a timeout
**
**   GSList **svcResponseList
**     This variable is set on a successful return to point to
**     service(s) found. Each element of this list is of type
**     ServiceRecordHandle (of the services which matched the search list)
**
**   uint16_t *maxAttrResponseByteCount
**     This is a pointer to a 16 bit integer, which is set to 
**     indicate the number of bytes of attributes returned. 
**     This pointer is set on successful return
**
*/
int __SDPServiceSearchAttributeRequest(
		BD_ADDR bdAddr,
		GSList *svcSearchList, 
		AttributeRequestType attrReqType,
		GSList *attrIDList,
		uint16_t maxAttrByteCount,
		GSList **svcResponseList,
		uint16_t *maxAttrResponseByteCount
		);


int __SDPServiceSearchPortRequest(BD_ADDR bda, UUID16Bit uuid16Bit);
int SDPServiceSearchRequest(
		SDPServerHandle srvHandle,
		GSList *svcSearchList, 
		uint16_t maxSvcRecordCount,
		GSList **svcResponseList,
		uint16_t *handleCountInResponse);
int SDPServiceAttributeRequest(
		SDPServerHandle srvHandle,
		ServiceRecordHandle svcHandle,
		AttributeRequestType attrReqType,
		GSList *attrIDList,
		uint16_t maxAttrIDByteCount,
		ServiceRecord	**_svcRec,
		uint16_t *maxAttrResponseByteCount);
int SDPServiceSearchAttributeRequest(
		SDPServerHandle srvHandle,
		GSList *svcSearchList, 
		AttributeRequestType attrReqType,
		GSList *attrIDList,
		uint16_t maxAttrByteCount,
		GSList **svcResponseList,
		uint16_t *maxAttrResponseByteCount);



/*
 ** In all the getXXX(svcRecHandle, XXX *xxx) functions below, 
 ** the XXX * is set to point to the value, should it exist
 ** and and E_OK is returned. If the value does not exist in the
 ** said service record a E_NOT_EXIST is returned 
 **
 ** In all the methods below, the memory management rules are
 ** simple. Don't free anything ! The pointer returned in the
 ** case of constructed types, is the real pointer to contents 
 ** in the ServiceRecord. For basic types, the address of the 
 ** local variable must be passed, which is set to the actual
 ** value
 */

#if 0
static inline int GetBasicAttribute(ServiceRecord *svcRec, ServiceAttributeIdentifier attrID, pAttrValue, fieldName)
{

	int status = E_NOT_EXIST;
	SDPData *pSDPData;
	pSDPData = getSDPData(svcRec, attrID);
	if (pSDPData != NULL) {
		*pAttrValue = pSDPData->sdpDataValue.fieldName;
			status = E_OK;
	}
	return status;
}
#endif
#define GetBasicAttribute(attrID, pAttrValue, fieldName)	\
								\
	int status = E_NOT_EXIST;				\
	SDPData *pSDPData;					\
	pSDPData = getSDPData(svcRec, attrID);			\
	if (pSDPData != NULL) { 				\
		*pAttrValue = pSDPData->sdpDataValue.fieldName;	\
			status = E_OK;  			\
	}  							\
	return status;                                                         


static inline int GetStringAttribute(ServiceRecord *svcRec, ServiceAttributeIdentifier attrID, char **pAttrValue)
{
	int status = E_NOT_EXIST;
	SDPData *pSDPData;
	pSDPData = getSDPData(svcRec, attrID);
	if (pSDPData != NULL) {
		int len = strlen(pSDPData->sdpDataValue.stringPtr);
			char *ptr = (char *) malloc(len + 1);
			memset(ptr, 0, (len+1));
			memcpy(ptr, pSDPData->sdpDataValue.stringPtr,len);
			*pAttrValue = ptr;
			status = E_OK;
	}
	return status;
}

static inline int GetUUIDSequenceAttribute(ServiceRecord *svcRec, ServiceAttributeIdentifier attrID, GSList **pAttrSeq)
{

	int status = E_NOT_EXIST;
	SDPData *pSDPData;
	SDPData *currSDPData;
	GSList	*list;

	*pAttrSeq = NULL;
	pSDPData = getSDPData(svcRec, attrID);
	if ((pSDPData != NULL) && ((pSDPData->dtd >= SequenceSize8) ||
				(pSDPData->dtd <= SequenceSize32)))  {
		for (list = pSDPData->sdpDataValue.dataSeq; list != NULL; list = g_slist_next(list)) {
			UUID *pUUID = (UUID *)malloc(sizeof(UUID));
			currSDPData = list->data;
			memset(pUUID, 0, sizeof(UUID));
			if ((currSDPData->dtd >= UUID_16) || (currSDPData->dtd <= UUID_128)) {
				*pUUID = currSDPData->sdpDataValue.uuid;
				*pAttrSeq = g_slist_append(*pAttrSeq, &currSDPData->sdpDataValue.uuid);
			} else {
				BTERROR("Incorrect dtd : 0x%x , expect UUID\n",	currSDPData->dtd); 
				break;
			}
		}
		status = E_OK;
	}
	return status;
}

int isSequenceData(SDPData *pSDPData);
int isAlternativeData(SDPData *pSDPData);

/*
 ** Get the AccessProtocols for the service record
 **
 ** If the value exists, the pointer "ap" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 ** 
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getAccessProtocols(ServiceRecord *svcRec, AccessProtocols **pap);


/*
 ** Extract the list of service classes that the service
 ** conforms to. 
 **
 ** If the value exists, the pointer "svcClassSeq" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** svcClassSeq when set, contains
 ** elements of ServiceClassIdentifer(UUID16Bit) ordered
 ** from most specific to the most generic
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceClassIDList(ServiceRecord *svcRec, GSList **svcClassSeq);


int getProtocolDescList(ServiceRecord *svcRec, GSList **svcClassSeq);


/*
 ** Extract the list of browse groups that the service
 ** belongs to. 
 **
 ** If the value exists, the pointer "browseGroupSeq" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** browseGroupSeq when set, contains
 ** elements of GroupID(UUID16Bit) 
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getBrowseGroupList(ServiceRecord *svcRec, GSList **browseGroupSeq);


/*
 ** Extract ServiceInfoTimeToLive of the service record.
 **
 ** If the value exists, the pointer "svcTTLInfo" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceInfoTimeToLive(ServiceRecord *svcRec, ServiceInfoTimeToLive *svcTTLInfo);

/*
 ** Extract ServiceRecordState attribute of the service record
 **
 ** If the value exists, the pointer "svcRecState" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceRecordState(ServiceRecord *svcRec, ServiceRecordState *svcRecState);

/*
 ** Extract ServiceAvailability attribute of the service record
 **
 ** If the value exists, the pointer "svcAvail" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceAvailability(ServiceRecord *svcRec, ServiceAvailability *svcAvail);

/*
 ** Extract ServiceID of the service record
 **
 ** If the value exists, the pointer "pUUID" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceID(ServiceRecord *svcRec, UUID *pUUID);


/*
 ** Extract GroupID of the service record
 **
 ** If the value exists, the pointer "pUUID" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getGroupID(ServiceRecord *svcRec, UUID *pUUID);


/*
 ** Check if this service record is a group descriptor
 ** or service descriptor. It is a group if GroupID is
 ** defined
 **
 ** If group, function returns 1 else 0
 */
unsigned char isGroup(ServiceRecord *svcRec);


/*
 ** Extract language attribute meta-data of the service record. 
 **
 ** If the value exists, the pointer "langSeq" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** For each language in the service record, the 
 ** GSList LangSeq returned has a struct of type
 ** LanguageAttributes. 
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getLanguageAttributes(ServiceRecord *svcRec, GSList **langSeq);


/*
 ** Extract the Bluetooth profile descriptor sequence of the
 ** service record.
 **
 ** If the value exists, the pointer "profDesc" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** Each element in the GSList returned is of type ProfileDescriptor
 ** which contains the UUID of the profile and it's version number
 ** which is encoded as major and minor in the high-order 8bits
 ** and low-order 8bits respectively of the 16bit VersionNumber
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getProfileDescriptor(ServiceRecord *svcRec, GSList **profDesc);

/*
 ** Extract ServiceName of the service record.
 **
 ** If the value exists, the pointer "pName" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceName(ServiceRecord *svcRec, char **pName);

/*
 ** Extract service description from the service record
 **
 ** If the value exists, the pointer "pDesc" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceDescription(ServiceRecord *svcRec, char **pDesc);

/*
 ** Extract service provider name from the service record
 **
 ** If the value exists, the pointer "pSvcProvider" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getServiceProviderName(ServiceRecord *svcRec, char **pSvcProvider);

/*
 ** Extract documentation URL from the service record
 **
 ** If the value exists, the pointer "pDocURL" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getDocumentationURL(ServiceRecord *svcRec, char **pDocURL);

/*
 ** Extract client-executable URL from the service record
 **
 ** If the value exists, the pointer "pClientURL" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getClientExecutableURL(ServiceRecord *svcRec, char **pClientURL);


/*
 ** Extract icon URL from the service record
 **
 ** If the value exists, the pointer "pIconURL" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 */
int getIconURL(ServiceRecord *svcRec, char **pIconURL);


/*
 ** Extract ServiceDatabaseState from the service record
 **
 ** Note that this is an attribute of the SDP server only and if
 ** it's value has changed from the previous value, then something
 ** in the SDP server DB has changed.
 **
 ** If the value exists, the pointer "svcDBState" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 **
 */
int getSDPServerServiceDatabaseState(ServiceRecord *svcRec, ServiceDatabaseState *svcDBState);

/*
 ** Extract SDP server supported version numbers 
 **
 ** Note that this is an attribute of the SDP server only and
 ** contains a list of uint16_t each of which represent the
 ** major and minor SDP version numbers supported by this server
 **
 ** If the value exists, the pointer "pVnumList" is set
 ** and E_OK is returned, else E_NOT_EXIST is returned
 **
 ** It is assumed that the attributes of service record
 ** were obtained from a previous service attribute request
 **
 */
int getSDPServerVersionNumber(ServiceRecord *svcRec, GSList **pVnumList);

void printServiceRecordById(ServiceRecord *svcRec);

/* -------------------------------------------- */

void makeUUIDFrom16Bits(UUID *pUUID, UUID16Bit value16Bit);
void makeUUIDFrom32Bits(UUID *pUUID, UUID32Bit value32Bit);
void makeUUIDFrom128Bits(UUID *pUUID, UUID128Bit *value128Bit);

/*
** UUID comparison function
**
** Returns 
** 0 if uuidValue1 == uuidValue2
** -1 if uuidValue1 < uidValue2
** 1 if uuidValue1 > uidValue2
**    
*/
gint compareUUID(gconstpointer p1, gconstpointer p2);

void convert16BitsTo128Bits(UUID *pUUID128Bit, UUID *pUUID16Bit);

UUID *convertTo128Bits(UUID *pUUID);

int extractUUID(char *buffer, UUID *pUUID, int *bytesScanned);

int generateUUIDPduForm(UUID *pUUID, char *buffer);

void printUUID(UUID *pUUID);  

int UUID2value(UUID *pUUID);
int UUID2strn(UUID *pUUID, char *str, size_t n);
#define MAX_LEN_UUID_STR 37		// The maximum string length
int ProtocolUUID2strn(UUID *pUUID, char *str, size_t n);
#define MAX_LEN_PROTOCOL_UUID_STR 8	// The maximum string length
int ServiceClassUUID2strn(UUID *pUUID, char *str, size_t n);
#define MAX_LEN_SERVICECLASS_UUID_STR 24	// The maximum string length
int ProfileDescriptorUUID2strn(UUID *pUUID, char *str, size_t n);
#define MAX_LEN_PROFILEDESCRIPTOR_UUID_STR 22	// The maximum string length


#endif //SDP_CLT_H
