/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: sdpsrv.h,v 1.7 2002/09/20 21:26:47 kds Exp $

   SDP service registration API definitions
 
   Fixes:
		Dmitry Kasatkin		: bug fixes, cleanup, re-arrangement, 
					  continuation state, mtu
*/
	 
#ifndef SDP_SRV_H
#define SDP_SRV_H

#include <affix/sdp.h>
//#include "record.h"
//#include "utils.h"

#define SDP_TCP_PORT 3400


SDPServerHandle SDPOpenLocalServerConnection(void);
BD_ADDR SDPGetLocalBDAddr(void);

/*
** Returns the internal state of the service record
** pointed to by svcRecHandle
*/
ServiceRecordInternalState getServiceRecordInternalState(ServiceRecord *svcRec);

/*
** NOTE that none of the setXXX() functions below will
** actually commit the changes to the DB, unless the
** commitServiceRecord() function is invoked.
**
** All changes are done to the local copy.
**
*/


/*
** Register a service record. 
**
** It is the responsbility of the Service Provider to 
** create the ServiceRecord first and set the
** attributes of the service record using the setXXX()
** methods defined in sdpsrv.h
**
** The service provider must then follow it up with a
** registerServiceRecord() to make the service record
** visible to SDP clients.
**
** Returns a 0 (and a pointer) to a service
** record if successful, else -1
*/
int __registerServiceRecord(ServiceRecord *svcRec);
int registerServiceRecord(SDPServerHandle srvHandle, ServiceRecord *svcRec);


/*
** Deletes a service record pointed to by
** svcRec. This deletes the SDP server's
** image as well as the local copy and
** the memory associated with the ServiceRecord
**
** Returns 0 if successful or -1 if there
** is a failure.
**
*/
int __deleteServiceRecord(ServiceRecord *svcRec);
int deleteServiceRecord(SDPServerHandle srvHandle, ServiceRecord *svcRec);
int __deleteServiceRecordByHandle(ServiceRecordHandle svcRecHandle);
int deleteServiceRecordByHandle(SDPServerHandle srvHandle, ServiceRecordHandle svcRecHandle);

/*
** Use this function to effect changes to an existing 
** ServiceRecord. Calling this function before a
** previous call to registerServiceRecord() will result
** in an error
*/
int __updateServiceRecord(ServiceRecord *svcRec);
int updateServiceRecord(SDPServerHandle srvHandle, ServiceRecord *svcRec);


/*
** NOTE that none of the setXXX() functions below will
** actually update the SDP server, unless the
** {register, update}ServiceRecord() function is invoked.
**
** Also note that the caller is responsible for memory
** management. All setXXX() involving constructed types
** such as AccessProtocols simply overwrite the previous
** pointer (if any). Old values and their memory mgmt is
** the responsibility of the caller.
**
*/

/*
** Add a Basic Attribute of the service
** pointed to by svcRec to its attribute list.
** the information necessary are the data type descriptor
** of the the service attribute, the value of the attribute itself 
** and the service attribute identifier.
** 
** Returns 0 if successful and -1 if there is a failure.
*/
int addBasicAttribute(ServiceRecord *svcRec, DataTypeDescriptor dtd, void *pValue,
		ServiceAttributeIdentifier attributeId
);

/*
** Set the information attributes of the service
** pointed to by svcRec. The set of attributes
** include service name, description and provider name
**
** Returns 0 if successful and -1 if there is
** a failure.
*/
int setInformationalAttributes(ServiceRecord *svcRec, char *serviceName,
		char *providerName, char *serviceDescription
);

/*
** This function sets the serviceClassID of the
** service pointed to by ServiceRecord,
** to the list specified by svcClassIDList.
** Note that the identifiers need to be in
** sorted order from the most specific to the
** most generic service class that this service
** conforms to.
**
** Using a value of NULL for svcClassIDList has
** effect of removing this attribute (if previously set)
** 
** This function replaces the existing ClassID
** list with a new one specified by svcClassIDList
** if one is present.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceClassID(ServiceRecord *svcRec, GSList *svcClassIDList);

SDPData *setServiceClassIDList(ServiceRecord *svc);
int addServiceClassID(SDPData *attr, UUID16Bit uuid);


/*
** This function sets the BrowseGroupList of the
** service pointed to by ServiceRecord,
** to the list specified by browseGroupSeq.
**
** A service can belong to one or more service
** groups and the list captures such group
** indentifiers (UUIDs)
**
** Using a value of NULL for browseGroupSeq has
** effect of removing this attribute (if previously set)
** 
** This function replaces the existing BrowseGroupList
** list with a new one specified by browseGroupSeq
** if one is present.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
SDPData *setBrowseGroupList(ServiceRecord *svc);
int addBrowseGroup(SDPData *attr, UUID16Bit uuid);


/*
** This function sets the access protocols
** of the service specified by ServiceRecordHandle.
** to the value specified in "accessProtocols"
**
** Note that if there are alternate mechanisms by 
** which the service is accessed, then they should 
** be specified as sequences and the "alternates"
** flag should be set.
**
** Using a value of NULL for acessProtocols has
** effect of removing this attribute (if previously set)
** 
** This function replaces the exisiting AccessProtocols
** structure (if any) with the new one specified.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
SDPData *setProtocolDescriptorList(ServiceRecord *svc);
/* for alternative protocols */
SDPData *addProtocolDescriptorList(SDPData *attr);
SDPData *addProtocolDescriptor(SDPData *attr, UUID16Bit uuid, 
		PortNumber portNumber, int portSize, VersionNumber version);


/*
** This function sets the "LanguageBase"
** attributes of the service record pointed
** to by svcRec to the value specified 
** in "langAttrList".
**
** "langAttrList" is a linked list of "LanguageAttributes"
** objects, one for each language in which user visible
** attributes are present in the service record.
**
** Using a value of NULL for langAttrList has
** effect of removing this attribute (if previously set)
** 
** This function replaces the exisiting LanguageAttributes
** structure (if any) with the new one specified.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
SDPData *setLanguageBaseAttributeIDList(ServiceRecord *svcRec);
int addLanguageBase(SDPData *attr, uint16_t lang, uint16_t encoding, uint16_t offset);
 

/*
** This function sets the "ServiceInfoTimeToLive"
** attribute of the service. This is the number
** of seconds that this service record is guaranteed
** not to change after the record was obtained by the
** client from the SDP server.
**
** It is not clear if this attribute
** is set by the SDP server or the owner or the
** service record. Just in case ..
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceInfoTimeToLive(ServiceRecord *svcRec, ServiceInfoTimeToLive svcTTL);


/*
** This function sets the "ServiceRecordState"
** attribute of the service. This attribute is
** guaranteed to change if there is any kind of
** modification to the service record. 
** 
** Again it is not clear if this attribute is set
** by the SDP server or the service provider. But
** this one seems more likely an attribute that
** is set by the service provider.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceRecordState(ServiceRecord *svcRec, ServiceRecordState svcRecState);


/*
** This function sets the "ServiceID"
** attribute of the service. 
** 
** This is the UUID of the service. 
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceID(ServiceRecord *svcRec, UUID16Bit svcUUID);


/*
** This function sets the "GroupID"
** attribute of a service descriptor 
** defining a Group
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setGroupID(ServiceRecord *svcRec, UUID16Bit groupUUID);


/*
** This function sets the "ServiceAvailability"
** attribute of the service pointed to svcRec
** to the value specified in svcAvail.
**
** Note that this represents the relative availability
** of the service 0x00 means compeletely unavailable to
** 0xFF meaning maximum availability.
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceAvailability(ServiceRecord *svcRec, ServiceAvailability svcAvail);


/*
** This function sets the "ProfileDescriptorList"
** attribute of the service record pointed
** to by svcRec to the value specified 
** in "profileDesc".
**
** Each element in the GSList is an object of type
** ProfileDescriptor which is a definition of the
** Bluetooth profile that this service conforms to.
**
** Using a value of NULL for profileDesc has
** effect of removing this attribute (if previously set)
** 
** This function replaces the exisiting BTProfileDescritorList
** structure (if any) with the new one specified.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
SDPData *setProfileDescriptorList(ServiceRecord *svc);
int addProfileDescriptor(SDPData *attr, UUID16Bit uuid, VersionNumber version);


/*
** Sets various URL attributes of the service
** pointed to by svcRec. The URL include
**
** ClientExecutableURL : a URL to the client's
**   platform specific (WinCE, PalmOS) executable
**   code that can be used to access this service.
**
** DocumentationURL : a URL pointing to service documentation
**
** IconURL : a URL to an icon that can be used to represent
**   this service.
**
** Note that you need to pass NULL for any URLs
** that you don't want to set or remove
**
** Returns 0 if successful and -1 if there is
** a failure.
**
*/
int setURLAttributes(ServiceRecord *svcRec, char *clientExecURL, char *docURL, char *iconURL);

/* ---------------------------------------- */

ServiceRecord *createRFCOMMServiceRecord(int port, int obex);
SDPData *createSDPData(DataTypeDescriptor dtd, void *pValue);
void addSDPDataSeq(SDPData *pSeq, SDPData *pSDPData);

#endif //SDP_SRV_H
